#!/usr/local/bin/tclsh8.0
#===========================================================================#
# SCRIPT  : satguisvr                                                       #
# PRODUCT : Small Application Toolkit.                                      #
# PURPOSE : Server process for SATGUI.                                      #
# AUTHOR  : Copyright (c) Adrian Davis 2000                                 #
# URL     : www.satisoft.com                                                #
#---------------------------------------------------------------------------#
# HISTORY : Oct00 Alpha1 - First alpha release.                             #
#===========================================================================#

#===========================================================================#
# Procedure to log message.                                                 #
#===========================================================================#

proc logMessage {Message} {
   set  Timestamp [exec date "+%Y/%m/%d %H:%M:%S"]
   puts "$Timestamp $Message"
}

#===========================================================================#
# Procedure to create socket connection.                                    #
#===========================================================================#

proc createConnection {Port} {
   global LOGIN

   logMessage "==================================================="
   logMessage "SATGUI SERVER STARTED - (Port $Port)"
   logMessage "---------------------------------------------------"

   set LOGIN(Server,Main) [socket -server acceptConnection $Port]
}

#===========================================================================#
# Procedure to accept socket connection.                                    #
#===========================================================================#

proc acceptConnection {Socket Address Port} {
   global LOGIN TITLE

   logMessage "Accept Connection ($Socket : $Address : $Port)"

   set LOGIN(Socket,$Socket)  [list $Address $Port]
   set LOGIN(New,$Socket)     "1"

   fconfigure $Socket -buffering line
   fileevent  $Socket readable [list readSocket $Socket]
   puts $Socket "initGUI {$TITLE}"
}

#===========================================================================#
# Procedure to read command from socket.                                    #
#===========================================================================#

proc readSocket {Socket} {
   global LOGIN PROFILES

   if {[eof $Socket] || [catch {gets $Socket Command}]} {
      logMessage "Closing ($LOGIN(Socket,$Socket))"
      catch {close $Socket}
      unset LOGIN(New,$Socket)
      unset LOGIN(Socket,$Socket)
      if [info exists LOGIN(User,$Socket)] {
         unset LOGIN(User,$Socket)
         unset LOGIN(Profile,$Socket)
         unset LOGIN(Printer,$Socket)
      }
   } else {
      set Action [lindex $Command 0]
      if {[string compare $Action login] == 0} {
         processLogin $Socket $Command
      } else {
         if {[string compare $Command ""] == 0} {return}
         if $LOGIN(New,$Socket) {
            logMessage "Invalid attempt to execute ($Command) from ($LOGIN(New,$Socket))"
            return
         }
         if [info exists PROFILES($Action,$LOGIN(Profile,$Socket))] {
            set Access $PROFILES($Action,$LOGIN(Profile,$Socket))
         } else {
            set Access 0
         }
         set UserAccess " $LOGIN(User,$Socket) $Access $LOGIN(Printer,$Socket) "
         if {[regsub { } $Command $UserAccess Command] != 1} {
            set Command "$Command $UserAccess"
         }
         processAction $Socket $Access $Command
      }
   }
}

#============================================================================#
# Read result from process.                                                   #
#============================================================================#

proc readProcessResult {Socket ProcessResult} {
   global RESULT

   if {[eof $ProcessResult] || [catch {gets $ProcessResult ResultLine}]} {
      catch {close $ProcessResult}
      puts $Socket "$RESULT($Socket)\034unlock"
   } else {
      set RESULT($Socket) "$RESULT($Socket)\034$ResultLine"
   }
}

#============================================================================#
# Procedure to process login.                                                #
#============================================================================#

proc processLogin {Socket Args} {
   global LOGIN PROFILES USERS

   set User     [lindex $Args 1]
   set Password [lindex $Args 2]

   if [info exists USERS($User)] {
      set UserDetails $USERS($User)
      if {[string compare [lindex $USERS($User) 1] $Password] == 0} {
         logMessage "--> Valid password"
         set LOGIN(New,$Socket)     0
         set LOGIN(User,$Socket)    $User
         set LOGIN(Profile,$Socket) [lindex $USERS($User) 2]
         set LOGIN(Printer,$Socket) [lindex $USERS($User) 3]
         set Access                 $PROFILES(main,$LOGIN(Profile,$Socket))

         processAction $Socket $Access [list main $LOGIN(User,$Socket) $Access $LOGIN(Printer,$Socket)]
      } else {
         logMessage "--> Invalid password"
         puts $Socket "setMessage {\"Invalid User/Password\" red};unlock"
      }
   } else {
      logMessage "--> Invalid user"
      puts $Socket "setMessage {\"Invalid User/Password\" red};unlock"
   }
}

#============================================================================#
# Procedure to read profiles file.                                           #
#============================================================================#

proc readProfiles {} {
   global APPLICATION SATHOME PROFILES

   set ProfileFile [open [file join $SATHOME config profiles]]

   foreach ProfileLine [split [read $ProfileFile] \n] {
      if {[regexp "(^\[ \t]*$)|(^#)" $ProfileLine] == 0} {
         set ProfileName [lindex $ProfileLine 0]
         set ProfileData [lrange $ProfileLine 1 end]

         foreach ProfileAccess $ProfileData {
            set Profile [split $ProfileAccess :]
            set PROFILES($ProfileName,[lindex $Profile 0]) [lindex $Profile 1]
         }
      }
   }
   close $ProfileFile
}

#============================================================================#
# To read users file.                                                        #
#============================================================================#

proc readUsers {} {
   global APPLICATION SATHOME USERS

   set UserFile [open [file join $SATHOME config users]]

   foreach UserLine [split [read $UserFile] \n] {
      if {[regexp "(^\[ \t]*$)|(^#)" $UserLine] == 0} {
         set UserData [split $UserLine :]
         set USERS([lindex $UserData 0]) $UserData
      }
   }
   close $UserFile
}

#============================================================================#
# Procedure to read screen file into array and send to client.               #
#============================================================================#

proc processAction {Socket Access Args} {
   global APPLICATION LOGIN PROFILES RESULT SATHOME

   set Action          [lindex $Args 0]
   set Parameters      [lrange $Args 1 end]
   set ScreenFileName  [file join $SATHOME screen $Action] 
   set ProcessFileName [file join $SATHOME process $Action]
   set Command         "| $ProcessFileName $Parameters"

   switch -- $Access {
      0 {puts $Socket "setMessage {\"Access Denied\" red};unlock";return}
      1 {set State disabled}
      2 {set State normal}
   }

#----------------------#
# Check screen exists. #
#----------------------#

   if [file exists $ScreenFileName] {
      set ScreenFile [open $ScreenFileName]
      set Screen     [read $ScreenFile]
      close $ScreenFile
      regsub -all "\n" $Screen "\034" Screen
      puts $Socket "setState $State\034$Screen"
   }

#--------------------------#
# Check if process exists. #
#--------------------------#

   if [file exists $ProcessFileName] {
      set RESULT($Socket) ""
      set ProcessResult [open $Command "r"]
      fconfigure $ProcessResult -buffering line
      fileevent  $ProcessResult readable [list readProcessResult $Socket $ProcessResult]
   }
}

#===========================================================================#
# Start.                                                                    #
#===========================================================================#

set Port   [lindex $argv 0]
set TITLE  [lindex $argv 1]


if {[info exists env(SATHOME)]} {
   set SATHOME $env(SATHOME)
} else {
   logMessage "Error: SATHOME not set"
   exit
}

logMessage "==================================================="
logMessage "STARTING SATGUI SERVER - ALPHA1"
logMessage "---------------------------------------------------"
logMessage "Home  : $SATHOME"
logMessage "Port  : $Port"
logMessage "Title : $TITLE"
logMessage "---------------------------------------------------"
logMessage "Reading Profiles..."

readProfiles

logMessage "Reading Users..."

readUsers

createConnection $Port

vwait Forever

#===========================================================================#
# End of script: satguisvr                                                  #
#===========================================================================#
